unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, pdScriptAPI;

type
  TfrmMain = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    lbInfo: TLabel;
    Label1: TLabel;
    Label2: TLabel;
    sbLoad: TButton;
    mScript: TMemo;
    Label3: TLabel;
    edParams: TEdit;
    Label4: TLabel;
    cbDelim: TComboBox;
    cbFunc: TComboBox;
    Label5: TLabel;
    mResult: TMemo;
    sbCall: TButton;
    cbIsFunction: TCheckBox;
    cbVar: TCheckBox;
    procedure cbFuncDropDown(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure sbLoadClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure sbCallClick(Sender: TObject);
    procedure cbFuncSelect(Sender: TObject);
  private
    { Private declarations }
    pScript:THandle; // temporary pointer to loaded script
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

procedure TfrmMain.FormCreate(Sender: TObject);
var
  LibFileName:string;
begin
  // Look for pdScript embedded interpreter placed near this demo program
  LibFileName:=ExpandFileName(ExtractFilePath(ParamStr(0))+'..\..\..\pdScriptE.dll');
  if not fileexists(LibFileName) then
  begin
    LibFileName:=ExtractFilePath(ParamStr(0))+'pdScriptE.dll';
    if not fileexists(LibFileName) then
      LibFileName:='pdScriptE.dll';
  end;

  // load pdScript embedded interpreter
  if pdScriptLoad(LibFileName)=0 then
  begin
    sbLoad.Enabled:=False;
    ShowMessage('pdScript embedded interpreter was not found!'#13#10'Please ensure that pdScriptE.dll '+
      'library is available in the system search path, '#13#10'or place it near this demo program. Then restart the demo again.');
  end;
  pScript:=0;
  cbFuncSelect(nil);
end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  // Release the previously loaded script
  if (pScript>1) and (pdScriptLibHandle<>0) then
  try
    Release_pdScript(pScript);
  except
    on E:Exception do
      ShowMessage(e.message);
  end;
end;

procedure TfrmMain.cbFuncDropDown(Sender: TObject);
var
  i,p:Integer;
  oldf,tmp:string;
begin
  // very very simple parser to provide a list of functions for testing
  oldf:=cbFunc.Text;
  cbFunc.Items.BeginUpdate;
  try
    cbFunc.Items.Clear;
    for i:=0 To mScript.Lines.Count-1 do
    begin
      tmp:=mScript.Lines[i];
      if Pos('procedure ',ansilowercase(tmp))=1 then
        Delete(tmp,1,10)
      else
      if Pos('function ',ansilowercase(tmp))=1 then
        Delete(tmp,1,9)
      else
        tmp:='';
      if Length(tmp)>0 then
      begin
        p:=Pos('(',tmp);
        if p=0 then
        begin
          p:=Pos(':',tmp);
          if p=0 then
            p:=Pos(';',tmp);
        end;
        if p>0 then
          cbFunc.Items.Add(Copy(tmp,1,p-1))
        else
          cbFunc.Items.Add(tmp);
      end;
    end;
  finally
    cbFunc.Items.EndUpdate;
    cbFunc.Text:=oldf;
  end;
end;

procedure TfrmMain.sbLoadClick(Sender: TObject);
var
  rep:string;
begin
  // Release the previously loaded script
  if pScript>1 then
  try
    Release_pdScript(pScript);
  except
    on E:Exception do
      ShowMessage(e.message);
  end;
  pScript:=StartInline_pdScriptEx(mScript.Lines.Text, '', '', rep, '', Application.MainForm.Handle);
  sbCall.Enabled:=pScript>1;
  if not sbCall.Enabled then
    ShowMessage('Script could not be loaded or executed!'#13#10#13#10+rep);
end;

procedure TfrmMain.cbFuncSelect(Sender: TObject);
var
  i:Integer;
  tmp,fnc:string;
begin
  // very very simple decision of what kind of execution should be used (you can use any kind of execution in your program)
  fnc:=ansilowercase(cbFunc.Text);
  for i:=0 To mScript.Lines.Count-1 do
  begin
    tmp:=mScript.Lines[i];
    if Pos('procedure '+fnc,ansilowercase(tmp))=1 then
    begin
      cbIsFunction.Checked:=False;
      cbVar.Checked:=Pos('var ',tmp)>0;
      break;
    end
    else
    if Pos('function '+fnc,ansilowercase(tmp))=1 then
    begin
      cbIsFunction.Checked:=True;
      cbVar.Checked:=Pos('var ',tmp)>0;
      break;
    end;
  end;
end;

procedure TfrmMain.sbCallClick(Sender: TObject);
var
  prm,res:string;
  ec:Integer;
begin
  // call selected function/procedure
  if cbFunc.Text='' then
    ShowMessage('Select a function first!')
  else
  if pScript>1 then
  begin
    // call the script function
    prm:=edParams.Text;
    res:='';
    if cbIsFunction.Checked then
    begin
      if cbVar.Checked then
        Execute_pdScriptFunctionEx(pScript,cbFunc.Text,prm,res,cbDelim.Text)
      else
        Execute_pdScriptFunction(pScript,cbFunc.Text,prm,res,cbDelim.Text);
    end
    else
    begin
      if cbVar.Checked then
        Execute_pdScriptProcedureEx(pScript,cbFunc.Text,prm,cbDelim.Text)
      else
        Execute_pdScriptFunction(pScript,cbFunc.Text,prm,cbDelim.Text);
    end;
    if cbVar.Checked then
      mResult.Lines.Text:=prm+#13#10+res
    else
      mResult.Lines.Text:=res;
  end
end;

end.
